#############################################################
#                                                           #
#  Cerberus Minimal Serpent Coupling Script                 #
#                                                           #
# Created by:  Ville Valtavirta              2024-04-11     #
# Modified by: Ville Valtavirta              2024-04-11     #
#                                                           #
#############################################################

import os

import numpy as np

import cerberus
from cerberus.solvers import CodeInput, Solver

# A bit more information

cerberus.LOG.set_verbosity(3)

# Path to Serpent executable

sss_exe_path = os.environ["SERPENT_EXE_PATH"]

# Set up and start up Serpent

sss_input = CodeInput(["./serpent_inputs/Serpent.inp", "./serpent_inputs/cool.ifc"])
sss = Solver("Serpent", sss_exe_path, add_params=["-port", "-omp", "8"])
sss.input = sss_input

sss.initialize()

#########################
# Picard iteration loop #
#########################

for iteration in range(3):

    # --- Tell Serpent to solve

    sss.solve()

    # --- Get output power

    tra = sss.get_output_field("sss_of_cool_power")
    tra.communicate()

    # --- Get powers (this is now a numpy array)

    powers = tra.value_vec

    ###########################
    # Calculate TH-solution   #
    ###########################

    # Coolant specific heat capacity

    cp = 4900 # J/(kg*K)

    # Coolant mass flow rate

    w = 0.3 # kg/s

    # Initialize cell boundary temperature and density to
    # inlet values

    T = np.zeros((11,)) + 520
    rho = np.zeros_like(T) + 813 #kg/m3

    # Calculate temperatures at (nz+1) axial nodes

    for i, P in enumerate(powers):

        # Calculate next temperature
        # T(i+1) = T(i) + P(i)/(w*cp)

        Tnext = T[i] + P/(w*cp)

        # Store new temperature

        T[i+1] = Tnext

        # Calculate next density based on temperature
        # Linear interpolation between
        # T0 = 520, rho0 = 813 kg/m3
        # T1 = 600, rho1 = 653 kg/m3

        rho[i+1] = 813 + (653 - 813)/(600.0 - 520.0)*(Tnext - 520.0)

    # --- Calculate cell values as averages of cell boundary values

    T_cell = 0.5*(T[0:-1] + T[1:])
    rho_cell = 0.5*(rho[0:-1] + rho[1:])

    ###########################
    # Update interface data   #
    ###########################

    tra = sss.get_input_field("sss_if_cool_temperature")
    tra.set_and_send(T_cell)

    tra = sss.get_input_field("sss_if_cool_density")
    tra.set_and_send(rho_cell)